@@ -1,9 +1,10 @@
+# SPDX-License-Identifier: GPL-2.0-only
 # This file is part of Scapy
+# See https://scapy.net/ for more information
 # Copyright (C) 2007, 2008, 2009 Arnaud Ebalard
 #               2015, 2016, 2017 Maxence Tury
 #               2019 Romain Perez
 #               2019 Gabriel Potter
-# This program is published under a GPLv2 license
 
 """
 Common TLS fields & bindings.
@@ -36,7 +37,7 @@ from scapy.layers.tls.crypto.cipher_aead import AEADTagError
 from scapy.layers.tls.crypto.cipher_stream import Cipher_NULL
 from scapy.layers.tls.crypto.common import CipherError
 from scapy.layers.tls.crypto.h_mac import HMACError
-import scapy.modules.six as six
+import scapy.libs.six as six
 if conf.crypto_valid_advanced:
     from scapy.layers.tls.crypto.cipher_aead import Cipher_CHACHA20_POLY1305
 
@@ -338,8 +339,11 @@ class TLS(_GenericTLSSessionInheritance):
                     return _TLSEncryptedContent
                 # Check TLS 1.3
                 if s and _tls_version_check(s.tls_version, 0x0304):
-                    if (s.rcs and not isinstance(s.rcs.cipher, Cipher_NULL) and
-                            byte0 == 0x17):
+                    _has_cipher = lambda x: (
+                        x and not isinstance(x.cipher, Cipher_NULL)
+                    )
+                    if (_has_cipher(s.rcs) or _has_cipher(s.prcs)) and \
+                            byte0 == 0x17:
                         from scapy.layers.tls.record_tls13 import TLS13
                         return TLS13
             if plen < 5:
@@ -782,10 +786,12 @@ _tls_alert_description = {
     50: "decode_error", 51: "decrypt_error",
     60: "export_restriction_RESERVED", 70: "protocol_version",
     71: "insufficient_security", 80: "internal_error",
-    90: "user_canceled", 100: "no_renegotiation",
+    86: "inappropriate_fallback", 90: "user_canceled",
+    100: "no_renegotiation", 109: "missing_extension",
     110: "unsupported_extension", 111: "certificate_unobtainable",
     112: "unrecognized_name", 113: "bad_certificate_status_response",
-    114: "bad_certificate_hash_value", 115: "unknown_psk_identity"}
+    114: "bad_certificate_hash_value", 115: "unknown_psk_identity",
+    116: "certificate_required", 120: "no_application_protocol"}
 
 
 class TLSAlert(_GenericTLSSessionInheritance):
